"""
GPT Researcher MCP Server

This script implements an MCP server for GPT Researcher, allowing AI assistants
to conduct web research and generate reports via the MCP protocol.
"""

import os
import json
import uuid
import logging
from typing import Dict, Any, Optional, AsyncGenerator
import asyncio
from dotenv import load_dotenv
from mcp.server.fastmcp import FastMCP
from gpt_researcher import GPTResearcher

# Load environment variables
load_dotenv()

from utils import (
    research_store,
    create_success_response,
    handle_exception,
    get_researcher_by_id,
    format_sources_for_response,
    format_context_with_sources,
    store_research_results,
    create_research_prompt
)

logging.basicConfig(
    level=logging.INFO,
    format='[%(asctime)s][%(levelname)s] - %(message)s',
)

logger = logging.getLogger(__name__)

def format_sse_event(event_name: str, data: Dict[str, Any]) -> str:
    """Formatiert Daten als SSE Event String."""
    json_data = json.dumps(data)
    return f"event: {event_name}\ndata: {json_data}\n\n"

# Initialize FastMCP server
mcp = FastMCP("GPT Researcher", host="0.0.0.0", port=8000, timeout_keep_alive=720)

# Initialize researchers dictionary
if not hasattr(mcp, "researchers"):
    mcp.researchers = {}


@mcp.resource("research://{topic}")
async def research_resource(topic: str) -> str:
    """
    Provide research context for a given topic directly as a resource.

    This allows LLMs to access web-sourced information without explicit function calls.

    Args:
        topic: The research topic or query

    Returns:
        String containing the research context with source information
    """
    # Check if we've already researched this topic
    if topic in research_store:
        logger.info(f"Returning cached research for topic: {topic}")
        return research_store[topic]["context"]

    # If not, conduct the research
    logger.info(f"Conducting new research for resource on topic: {topic}")

    # Initialize GPT Researcher
    researcher = GPTResearcher(topic)

    try:
        # Conduct the research
        await researcher.conduct_research()

        # Get the context and sources
        context = researcher.get_research_context()
        sources = researcher.get_research_sources()
        source_urls = researcher.get_source_urls()

        # Format with sources included
        formatted_context = format_context_with_sources(topic, context, sources)

        # Store for future use
        store_research_results(topic, context, sources, source_urls, formatted_context)

        return formatted_context
    except Exception as e:
        return f"Error conducting research on '{topic}': {str(e)}"


@mcp.tool()
async def deep_research(query: str) -> AsyncGenerator[str, None]:
    """
    Conduct a web deep research on a given query using GPT Researcher.
    Use this tool when you need time-sensitive, real-time information like stock prices, news, people, specific knowledge, etc.

    Args:
        query: The research query or topic

    Returns:
        Dict containing research status, ID, and the actual research context and sources
        that can be used directly by LLMs for context enrichment
    """
    logger.info(f"Starting streaming deep research on query: {query}...")
    research_id = str(uuid.uuid4())

    try:
        yield format_sse_event(
            "message",
            {"research_id": research_id, "query": query, "status": "Research initiated"}
        )

        # Initialize GPT Researcher
        # In neueren GPTResearcher Versionen wird der query beim Initialisieren übergeben
        researcher = GPTResearcher(query=query, report_type="research_report", config_path=None)
        # Alternativ, falls deine Version es anders handhabt: researcher = GPTResearcher(query)

        mcp.researchers[research_id] = researcher # Speichere früh, falls benötigt

        yield format_sse_event(
            "message",
            {"research_id": research_id, "status": "GPT Researcher initialized. Starting information gathering."}
        )
        await asyncio.sleep(0.1) # Kurze Pause, damit das Event sicher gesendet wird

        # Simuliertes Update: Beginn der Recherche
        yield format_sse_event(
            "message",
            {"research_id": research_id, "status": "Starting web crawling and source analysis...", "progress": 10}
        )

        # Der eigentliche langlaufende Prozess
        await researcher.conduct_research() # Dieser Aufruf blockiert für die Dauer der Recherche

        # Simuliertes Update: Recherche abgeschlossen, beginne mit der Verarbeitung
        yield format_sse_event(
            "message",
            {"research_id": research_id, "status": "Web crawling finished. Processing and consolidating information...", "progress": 70}
        )
        await asyncio.sleep(0.1)

        logger.info(f"Core research completed for ID: {research_id}. Fetching results...")

        # Get the research context and sources
        context = researcher.get_research_context()
        sources = researcher.get_research_sources() # Dies sind strukturierte Source-Objekte
        source_urls = researcher.get_source_urls() # Dies sind nur die URLs

        # Store in the research store for the resource API (optional, je nach Logik)
        # Überlege, ob formatted_context hier gebraucht wird, wenn der Client die Rohdaten bekommt
        store_research_results(query, context, sources, source_urls, context) # Vereinfacht für den Store

        # Finale Daten vorbereiten
        final_data_payload = {
            "research_id": research_id,
            "query": query,
            "status": "Research completed successfully",
            "source_count": len(sources),
            "context": context,
            "sources": format_sources_for_response(sources),
            "source_urls": source_urls
        }

        # Sende das finale Ergebnis als 'tool_result' Event
        yield format_sse_event("message", final_data_payload)
        logger.info(f"Sent final research result for ID: {research_id}")

    except Exception as e:
        logger.error(f"Error during deep_research for query '{query}': {e}", exc_info=True)
        # Sende ein Fehler-Event an den Client
        error_payload = {
            "research_id": research_id, # Kann None sein, wenn Fehler sehr früh auftritt
            "query": query,
            "status": "Error occurred",
            "error_message": str(e),
            "error_details": "Check server logs for more information."
        }
        yield format_sse_event("message", error_payload) # Eigener Event-Typ für Fehler
        # Du könntest hier auch handle_exception verwenden, wenn es ein SSE-Event zurückgibt
        # oder die Exception weiter werfen, wenn FastMCP das besser handhabt.

@mcp.tool()
async def quick_search(query: str) -> AsyncGenerator[str, None]:
    """
    Perform a quick web search on a given query and return search results with snippets.
    This optimizes for speed over quality and is useful when an LLM doesn't need in-depth
    information on a topic.

    Args:
        query: The search query

    Returns:
        Dict containing search results and snippets
    """
    logger.info(f"Performing quick search on query: {query}...")

    # Generate a unique ID for this search session
    search_id = str(uuid.uuid4())

    # Initialize GPT Researcher
    researcher = GPTResearcher(query)

    try:
        yield format_sse_event(
            "message",
            {"research_id": search_id, "query": query, "status": "Research initiated"}
        )

        # Perform quick search
        search_results = await researcher.quick_search(query=query)
        mcp.researchers[search_id] = researcher
        logger.info(f"Quick search completed for ID: {search_id}")

        final_data_payload = {
            "search_id": search_id,
            "query": query,
            "result_count": len(search_results) if search_results else 0,
            "search_results": search_results
        }

        # Sende das finale Ergebnis als 'tool_result' Event
        yield format_sse_event("message", final_data_payload)
        logger.info(f"Sent final research result for ID: {search_id}")
    except Exception as e:
        logger.error(f"Error during deep_research for query '{query}': {e}", exc_info=True)
        # Sende ein Fehler-Event an den Client
        error_payload = {
            "research_id": search_id, # Kann None sein, wenn Fehler sehr früh auftritt
            "query": query,
            "status": "Error occurred",
            "error_message": str(e),
            "error_details": "Check server logs for more information."
        }
        yield format_sse_event("message", error_payload)


@mcp.tool()
async def write_report(research_id: str, custom_prompt: Optional[str] = None) -> AsyncGenerator[str, None]:
    """
    Generate a report based on previously conducted research.

    Args:
        research_id: The ID of the research session from conduct_research
        custom_prompt: Optional custom prompt for report generation

    Returns:
        Dict containing the report content and metadata
    """
    success, researcher, error = get_researcher_by_id(mcp.researchers, research_id)
    if not success:
        yield format_sse_event("message", error)

    logger.info(f"Generating report for research ID: {research_id}")

    try:
        # Generate report
        report = await researcher.write_report(custom_prompt=custom_prompt)

        # Get additional information
        sources = researcher.get_research_sources()
        costs = researcher.get_costs()

        final_data_payload = {
            "report": report,
            "source_count": len(sources),
            "costs": costs
        }

        # Sende das finale Ergebnis als 'tool_result' Event
        yield format_sse_event("message", final_data_payload)
        logger.info(f"Sent final research result for ID: {research_id}")
    except Exception as e:
        logger.error(f"Error during deep_research for query: {e}", exc_info=True)
        # Sende ein Fehler-Event an den Client
        error_payload = {
            "research_id": research_id,
            "status": "Error occurred",
            "error_message": str(e),
            "error_details": "Check server logs for more information."
        }
        yield format_sse_event("message", error_payload)


@mcp.tool()
async def get_research_sources(research_id: str) -> AsyncGenerator[str, None]:
    """
    Get the sources used in the research.

    Args:
        research_id: The ID of the research session

    Returns:
        Dict containing the research sources
    """
    success, researcher, error = get_researcher_by_id(mcp.researchers, research_id)
    if not success:
        yield format_sse_event("message", error)

    sources = researcher.get_research_sources()
    source_urls = researcher.get_source_urls()

    final_data_payload = {
        "sources": format_sources_for_response(sources),
        "source_urls": source_urls
    }

    # Sende das finale Ergebnis als 'tool_result' Event
    yield format_sse_event("message", final_data_payload)
    logger.info(f"Sent final research result for ID: {research_id}")


@mcp.tool()
async def get_research_context(research_id: str) -> Dict[str, Any]:
    """
    Get the full context of the research.

    Args:
        research_id: The ID of the research session

    Returns:
        Dict containing the research context
    """
    success, researcher, error = get_researcher_by_id(mcp.researchers, research_id)
    if not success:
        return error

    context = researcher.get_research_context()

    return create_success_response({
        "context": context
    })


@mcp.prompt()
def research_query(topic: str, goal: str, report_format: str = "research_report") -> str:
    """
    Create a research query prompt for GPT Researcher.

    Args:
        topic: The topic to research
        goal: The goal or specific question to answer
        report_format: The format of the report to generate

    Returns:
        A formatted prompt for research
    """
    return create_research_prompt(topic, goal, report_format)


def run_server():
    """Run the MCP server using FastMCP's built-in event loop handling."""
    # Check if API keys are set
    if not os.getenv("OPENAI_API_KEY"):
        logger.error("OPENAI_API_KEY not found. Please set it in your .env file.")
        return

    # Add startup message
    logger.info("Starting GPT Researcher MCP Server...")
    print("🚀 GPT Researcher MCP Server starting... Check researcher_mcp_server.log for details")

    # Let FastMCP handle the event loop
    try:
        mcp.run("sse")
        # Note: If we reach here, the server has stopped
        logger.info("MCP Server has stopped")
    except Exception as e:
        logger.error(f"Error running MCP server: {str(e)}")
        print(f"❌ MCP Server error: {str(e)}")
        return

    print("✅ MCP Server stopped")


if __name__ == "__main__":
    # Use the non-async approach to avoid asyncio nesting issues
    run_server()